/*******************************************************************
    A telegram bot for your ESP8266 that controls the
    onboard LED. The LED in this example is active low.

    Parts:
    D1 Mini ESP8266 * - http://s.click.aliexpress.com/e/uzFUnIe
    (or any ESP8266 board)

      = Affilate

    If you find what I do useful and would like to support me,
    please consider becoming a sponsor on Github
    https://github.com/sponsors/witnessmenow/


    Written by Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow
 *******************************************************************/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>

// Wifi network station credentials
#define WIFI_SSID "il nome (SSID) della tua rete WiFi"
#define WIFI_PASSWORD "la password della tua rete WiFi"
// Telegram BOT Token (Get from Botfather)
#define BOT_TOKEN "il token che ti ha fornito il Botfather (codice lungo)"

const unsigned long BOT_MTBS = 1000; // mean time between scan messages

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure secured_client;
UniversalTelegramBot bot(BOT_TOKEN, secured_client);
unsigned long bot_lasttime; // last time messages' scan has been done

const int led1Pin = 14;
const int led2Pin = 12;
const int led3Pin = 13;
int led1Status = 1;
int led2Status = 1;
int led3Status = 1;

void handleNewMessages(int numNewMessages)
{
  Serial.print("handleNewMessages ");
  Serial.println(numNewMessages);

  for (int i = 0; i < numNewMessages; i++)
  {
    String chat_id = bot.messages[i].chat_id;
    String text = bot.messages[i].text;

    String from_name = bot.messages[i].from_name;
    if (from_name == "")
      //from_name = "Guest";
      from_name = "il codice di 10 cifre abbinato al tuo canale";

    if (text == "led1on" or text == "Led1on")
    {
      digitalWrite(led1Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led1Status = 1;
      bot.sendMessage(chat_id, "Led1 is ON", "");
    }

    if (text == "led1off" or text == "Led1off")
    {
      led1Status = 0;
      digitalWrite(led1Pin, LOW); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Led1 is OFF", "");
    }

    if (text == "led2on" or text == "Led2on")
    {
      digitalWrite(led2Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led2Status = 1;
      bot.sendMessage(chat_id, "Led2 is ON", "");
    }

    if (text == "led2off" or text == "Led2off")
    {
      led2Status = 0;
      digitalWrite(led2Pin, LOW); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Led2 is OFF", "");
    }


    if (text == "led3on" or text == "Led3on")
    {
      digitalWrite(led3Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led3Status = 1;
      bot.sendMessage(chat_id, "Led3 is ON", "");
    }

    if (text == "led3off" or text == "Led3off")
    {
      led3Status = 0;
      digitalWrite(led3Pin, LOW); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Led3 is OFF", "");
    }

    if (text == "ledon" or text == "Ledon")
    {
      digitalWrite(led1Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led1Status = 1;
      bot.sendMessage(chat_id, "Led1 is ON", "");
      digitalWrite(led2Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led2Status = 1;
      bot.sendMessage(chat_id, "Led2 is ON", "");
      digitalWrite(led3Pin, HIGH); // turn the LED on (HIGH is the voltage level)
      led3Status = 1;
      bot.sendMessage(chat_id, "Led3 is ON", "");
    }

    if (text == "ledoff" or text == "Ledoff")
    {
      //ledStatus = 0;
      digitalWrite(led1Pin, LOW); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Led1 is OFF", "");
      led1Status = 0;
      digitalWrite(led2Pin, LOW); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Led2 is OFF", "");
      led2Status = 0;
      digitalWrite(led3Pin, LOW); // turn the LED off (LOW is the voltage level)
      led3Status = 0;
      bot.sendMessage(chat_id, "Led3 is OFF", "");
    }


    if (text == "status" or text == "Status")
    {
      if (led1Status == 1)
      {
        bot.sendMessage(chat_id, "Led 1 is ON", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Led 1 is OFF", "");
      }
      if (led2Status == 1)
      {
        bot.sendMessage(chat_id, "Led 2 is ON", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Led 2 is OFF", "");
      }
      if (led3Status == 1)
      {
        bot.sendMessage(chat_id, "Led 3 is ON", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Led 3 is OFF", "");
      }
    }

    if (text == "help" or text == "Help")
    {
      String welcome = "Welcome to Universal Arduino Telegram Bot library, " + from_name + ".\n";
      welcome += "ledon :   switchs all the Leds ON.\n";
      welcome += "ledoff :  switchs all the Leds OFF\n";
      welcome += "led1on :  switchs the Led 1 ON\n";
      welcome += "led1off : switchs the Led 1 OFF\n";
      welcome += "led2on :  switchs the Led 2 ON\n";
      welcome += "led2off : switchs the Led 2 OFF\n";
      welcome += "led3on :  switchs the Led 3 ON\n";
      welcome += "led3off : switchs the Led 3 OFF\n";
      welcome += "status :  status of the leds ON/OFF\n";      
      bot.sendMessage(chat_id, welcome, "Markdown");
    }
  }
}


void setup()
{
  Serial.begin(115200);
  Serial.println();

  pinMode(led1Pin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(led1Pin, HIGH); // initialize pin as off (active LOW)
  pinMode(led2Pin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(led2Pin, HIGH); // initialize pin as off (active LOW)
  pinMode(led3Pin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(led3Pin, HIGH); // initialize pin as off (active LOW)// attempt to connect to Wifi network:
  configTime(0, 0, "pool.ntp.org");      // get UTC time via NTP
  secured_client.setTrustAnchors(&cert); // Add root certificate for api.telegram.org
  Serial.print("Connecting to Wifi SSID ");
  Serial.print(WIFI_SSID);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

  // Check NTP/Time, usually it is instantaneous and you can delete the code below.
  Serial.print("Retrieving time: ");
  time_t now = time(nullptr);
  while (now < 24 * 3600)
  {
    Serial.print(".");
    delay(100);
    now = time(nullptr);
  }
  Serial.println(now);
}

void loop()
{
  if (millis() - bot_lasttime > BOT_MTBS)
  {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages)
    {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }

    bot_lasttime = millis();
  }
}
